const std = @import("std");
const net = std.net;

const Listener = @import("root.zig").Listener(Context);
const Request = Listener.Request;
const Response = Listener.Response;

const Context = struct {};

pub fn main() !void {
    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    defer _ = gpa.deinit();
    const allocator = gpa.allocator();

    const address = net.Address.initIp4(.{ 127, 0, 0, 1 }, 8080);
    const listener = try Listener.init(.{
        .address = address,
        .allocator = allocator,
        .handler = handle,
        .errorHandler = handleError,
    });
    defer listener.deinit();

    try std.io.getStdErr().writer().print("listening at {}\n", .{address});
    try listener.listen();
}

fn handle(req: *const Request) anyerror!Response {
    if (std.mem.eql(u8, req.uri.path.percent_encoded, "/")) {
        return .{ .body = "home!" };
    } else {
        return .{ .body = "somewhere else!" };
    }
}

fn handleError(_: *const Request, err: [:0]const u8) anyerror!Response {
    return .{
        .body = "oops! something went wrong on the server side.",
        .options = .{
            .status = .internal_server_error,
            .reason = err,
        },
    };
}
